#!/usr/bin/python

"""
HTTP access to BTA TCS through "/tcs/ctrl/bta_send.cgi"
"""
__author__ = "Vladimir Shergin <http://www.sao.ru/hq/vsher/>"

import sys, re, socket
import http.client, urllib.request, urllib.parse, urllib.error

latitude=43.6548
server_url = None  # networked HTTP server
psw="suser"
host="ann"
server = None
headers = None
level = 0
mode = ""
stime = 0.0
mjd = 0.0
ra = 0.0
dec = 0.0
az = 0.0
zd = 0.0
stat = "Off"

def bta_ok():
    "Internal function"
    global server
    return isinstance(server,http.client.HTTPConnection)

def bta_init(thost=None,pswd=None):
    """ Create an object to represent HTTP server
    and try to connect to it.
    """
    global server,server_url,headers,host,psw,level
    if thost!=None: host=thost
    if pswd!=None: psw=pswd
    if not bta_ok():
        if server_url == None:
            if host.find('.')>=0:
                server_url = "http://"+host
                server = http.client.HTTPConnection(host, timeout=10)
            else:
                server_url = "http://"+host+".sao.ru"
                server = http.client.HTTPConnection(host+".sao.ru", timeout=10)
        if headers == None:
            headers = {"Connection": "Keep-Alive", "Content-type": "application/x-www-form-urlencoded","Accept": "text/plain"}
        print("Try to connect to BTA TCS (URL: "+server_url+")")
#        server.set_debuglevel(1)
        return bta_get()
    if level==0:
        print("Try to reconnect to BTA TCS (URL: "+server_url+")")
        return bta_get() 
    return True
def bta_level():
    global level
    return level

def bta_get():
    global server,headers,psw,level
    global stime, mjd, mode, ra, dec, az, zd
    if not bta_ok(): return False
    params = urllib.parse.urlencode({"Pass": psw, "Cmd": "Get"})
    level=0
    try:
        server.connect()
        server.request("POST", "/tcs/ctrl/bta_send.cgi", params, headers)
        resp = server.getresponse()
        if resp.status != 200:
            print(resp.status, resp.reason)
            return False
        state = "Err"
        ll = resp.read().decode().split('\n')
        for i in range(len(ll)):
            l = str(ll[i])
            if l.count('=')==1:
                nv = l.split('=')
                nam = str(nv[0])
                val = str(nv[1])
                if   nam == "Level":   level = int(val)
                elif nam == "Stime":   stime = float(val)%24
                elif nam == "MJD":     mjd = float(val)
                elif nam == "TelMode": mode = val
                elif nam == "RA":      ra = float(val)
                elif nam == "Dec":     dec = float(val)
                elif nam == "A":       az = float(val)
                elif nam == "Z":     zd = float(val)
                elif nam == "State":   state = val
            else:
                print(l)
        if state == "Ok":
            return True
        else:
            return False
    except socket.error as serr:
#        print("Network Error: %s" % serr.strerror)
        print("Network Error:", serr)
    except http.client.HTTPException as err:
        print("HTTP error %s" % str(err))
    try:
        server.close()
    finally:
        del server
        server = None
        return False
        
def bta_status():
    "Get BTA TCS current state"
    global level, mode
    if not bta_ok():
        if not bta_init():
            return "Off"
    elif not bta_get():
        return "Off"
    if level<1:
        print("User's level too low!")
        return "Error"
    return mode

def bta_is_on():
    "BTA TCS and telescope are ready to work?"
    global stat
    stat = bta_status()
    return not (stat.startswith("Of") or stat.startswith("Err"))

def bta_send_to(ra,dec,epoch=2000.0,objname=None,cmd="Send"):
    """ Send RA/Dec coordinates to BTA TCS
    RA in hours (hh.hhhhh) Dec in degrees (dd.ddddd)
    Return True if coordinates is Ok
    """
    global server,headers,psw,level,stime
    if not bta_is_on():
       print("Can't send RA/Dec! BTA TCS is \"Off\" (or wrong passwd)")
       return False
    if level<2:
        print("User's level %d - too low!" % level)
        return False
    if dec>90. or dec <= latitude-90.:
        print("Unreachable Declination %f" % dec)
        return False
    if ra>=24.: ra -= 24.
    if ra<0: ra += 24.
    reqst = "RA="+str(ra)+"&Decl="+str(dec)+" Epoch="+str(epoch)
    if objname == None:
        params = urllib.parse.urlencode({"Pass":psw, "Cmd":cmd, "RA":str(ra), "Dec":str(dec), "Epoch":str(epoch)})
    else:
        params = urllib.parse.urlencode({"Pass":psw, "Cmd":cmd, "RA":str(ra), "Dec":str(dec), "Epoch":str(epoch), "Name":objname})
    try:
        server.connect()
        server.request("POST", "/tcs/ctrl/bta_send.cgi", params, headers)
        resp = server.getresponse()
        if resp.status != 200:
            print(resp.status, resp.reason)
            return False
        state = "Err"
        ll = resp.read().decode().split('\n')
        for i in range(len(ll)):
            l = str(ll[i])
            if l.count('=')==1:
                nv = l.split('=')
                nam = str(nv[0])
                val = str(nv[1])
                if nam == "State":   state = val
            else:
                print(l)
        if state == "Ok":
            return True
        else:
            return False
    except socket.error as serr:
#        print("Network Error: %s" % serr.strerror)
        print("Network Error:", serr)
    except http.client.HTTPException as err:
        print("HTTP error %s" % str(err))
    try:
        server.close()
    finally:
        del server
        server = None
        return False

def bta_slew_to(ra,dec,epoch=2000.0,objname=None):
    """ Slew telescope to RA/Dec coordinates.
    RA in hours (hh.hhhhh) Dec in degrees (dd.ddddd)
    Returns True if telescope and coordinates are Ok and slewing begins.
    """
    global stat, level
    if not bta_send_to(ra,dec,epoch=epoch,objname=objname,cmd="Slew"):
        if(stat.startswith("Of") or stat.startswith("Err")):
            print("Can't slew! BTA TCS is \"Off\" (or wrong passwd)")
        elif level<4:
            print("Can't slew! User's level %d - too low!" % level)
        return False
    if level<4:
        print("Can't slew! User's level %d - too low!" % level)
    return True

#-------------------------------------------
# Test code
import time
if __name__ == "__main__":
    res = bta_init()
    print(res, level, stime, mjd, mode, ra, dec, az, zd)
    time.sleep(6.0)
    if res:
        res = bta_send_to(20.0,10.0,objname="Test_GCN_1")
        print(res, level, stime, mjd, mode, ra, dec, az, zd)
        time.sleep(6.0)
        res = bta_slew_to(7.0,10.0,objname="Test_GCN_2")
        print(res)
        time.sleep(6.0)
        res = bta_get()
        print(res, level, stime, mjd, mode, ra, dec, az, zd)
