#!/usr/bin/env python

"""
Payload handler for pygcn package by Leo Singer <leo.singer@ligo.org>.
"""
__author__ = "Vladimir Shergin <http://www.sao.ru/hq/vsher/>"

import sys
import time
import gcn
import gcn.notice_types as n
from gcn_alerts import *
from gcn_ztcs import hour2hms, degr2dms

gcn_host1 = "209.208.78.170"    # on Atlantic.net cloud service
gcn_host2 = "50.116.49.68"      # on Linode.com cloud service
gcn_host3 = "68.169.57.253"     # on eApps.com cloud service
gcn_host4 = "45.58.43.186"      # VOEvent 2.0 new (24 Nov 2015) server
gcn_port = 8099

import datetime
import logging

logging.basicConfig(level=logging.INFO)

use_tests = False # Use Test messages for practicing & debugging purpose
def gcn_use_tests():
    global use_tests
    use_tests = True

def gcn_save_xml(ntype,xmltext):
    fname = "gcn_voevent_type%d.xml" % ntype
    with open(fname, "w") as f:
        f.write(xmltext)
        f.close()

grb_ID = -777  # last GRB ident number  

def gcn_handler(payload, root):
    global use_tests, grb_ID
    test = False
    type = 0
    type = int(root.find("./What/Param[@name='Packet_Type']").attrib['value'])
    if type < 1 or type > 199:
        print("Unknown GCN Packet Type: %d" % type)
        sys.stdout.flush()
        return
    if "role" in root.attrib:
        role = root.attrib["role"]
        test = role.startswith('test')
    else:
        role = "unknown"
    try:
        description = root.find("./What/Description").text.strip()
    except AttributeError:
#        print "GCN VOEvent Type %d - no Description?!" % type
        description = "No Description"
    scname = "???"
    if type == n.IPN_RAW or type == n.IPN_POS:
        if root.find("./What/Group[@name='Misc']/Param[@name='Ulysses_contributed']").attrib['value'].lower().startswith('true'):
            scname = "IPN(Ulysses)"
        elif root.find("./What/Group[@name='Misc']/Param[@name='Mars_Odyssey_HEND_contributed']").attrib['value'].lower().startswith('true'):
            scname = "IPN(Mars_Odyssey_HEND)"
        elif root.find("./What/Group[@name='Misc']/Param[@name='Mars_Odyssey_GRS_contributed']").attrib['value'].lower().startswith('true'):
            scname = "IPN(Mars_Odyssey_GRS)"
        elif root.find("./What/Group[@name='Misc']/Param[@name='RHESSI_contributed']").attrib['value'].lower().startswith('true'):
            scname = "IPN(RHESSI)"
        elif root.find("./What/Group[@name='Misc']/Param[@name='KONUS_Wind_contributed']").attrib['value'].lower().startswith('true'):
            scname = "IPN(KONUS_Wind)"
        elif root.find("./What/Group[@name='Misc']/Param[@name='Swift_BAT_contributed']").attrib['value'].lower().startswith('true'):
            scname = "IPN(Swift_BAT)"
        elif root.find("./What/Group[@name='Misc']/Param[@name='Messenger_GRNS_contributed']").attrib['value'].lower().startswith('true'):
            scname = "IPN(Messenger_GRNS)"
        elif root.find("./What/Group[@name='Misc']/Param[@name='Suzaku_WAM_contributed']").attrib['value'].lower().startswith('true'):
            scname = "IPN(Suzaku_WAM)"
        elif root.find("./What/Group[@name='Misc']/Param[@name='INTEGRAL_SPIACS_contributed']").attrib['value'].lower().startswith('true'):
            scname = "IPN(INTEGRAL_SPIACS)"
        elif root.find("./What/Group[@name='Misc']/Param[@name='SuperAGILE_contributed']").attrib['value'].lower().startswith('true'):
            scname = "IPN(SuperAGILE)"
        elif root.find("./What/Group[@name='Misc']/Param[@name='AGILE_GRID_contributed']").attrib['value'].lower().startswith('true'):
            scname = "IPN(AGILE_GRID)"
        elif root.find("./What/Group[@name='Misc']/Param[@name='AGILE_MCAL_contributed']").attrib['value'].lower().startswith('true'):
            scname = "IPN(AGILE_MCAL)"
        elif root.find("./What/Group[@name='Misc']/Param[@name='Fermi_GBM_contributed']").attrib['value'].lower().startswith('true'):
            scname = "IPN(Fermi_GBM)"
        elif root.find("./What/Group[@name='Misc']/Param[@name='Fermi_LAT_contributed']").attrib['value'].lower().startswith('true'):
            scname = "IPN(Fermi_LAT)"
        else: scname = "IPN(???)"

    if type == n.IPN_RAW or type == n.INTEGRAL_SPIACS or \
       type == n.SWIFT_BAT_GRB_ALERT or \
       type == n.FERMI_GBM_ALERT or type == n.FERMI_GBM_FLT_POS or \
       type == n.FERMI_GBM_GND_POS or type == n.FERMI_GBM_FIN_POS or \
       type == n.FERMI_LAT_POS_UPD or type == n.FERMI_GBM_SUBTHRESH or \
       type == n.MAXI_UNKNOWN:
        if type == n.FERMI_GBM_SUBTHRESH:
            trigID = int(root.find("./What/Param[@name='Trans_Num']").attrib['value'])
        else:
            trigID = int(root.find("./What/Param[@name='TrigID']").attrib['value'])
        elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Time/TimeInstant/ISOTime")
        isotime = elm.text.strip()
        dtime = datetime.datetime.strptime(isotime, "%Y-%m-%dT%H:%M:%S.%f")
        elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Position2D/Error2Radius")
        err = float(elm.text)
        signif = 0.
        lcurl = "no"
        mapurl = "no"
        if type == n.INTEGRAL_SPIACS:
            scname = "INTEGRAL"
            test = root.find("./What/Group[@name='Test_mpos']/Param[@name='Test_Notice']").attrib['value'].lower().startswith('true')
            signif = float(root.find("./What/Param[@name='Inten_Sigma']").attrib['value'])
        elif type == n.SWIFT_BAT_GRB_ALERT:
            scname = "Swift-BAT"
            signif = float(root.find("./What/Param[@name='Rate_Signif']").attrib['value'])
        elif type == n.MAXI_UNKNOWN: scname = "MAXI-new"
        elif type == n.MAXI_KNOWN: scname = "MAXI-known"
        elif not type == n.IPN_RAW:
            scname = "Fermi-GBM"
            if type == n.FERMI_GBM_ALERT:
                signif = float(root.find("./What/Param[@name='Trig_Signif']").attrib['value'])
                scname = scname + "(Alert)"
                lcurl = root.find("./What/Param[@name='LightCurve_URL']").attrib['value']
            elif type == n.FERMI_GBM_FLT_POS:
                signif = float(root.find("./What/Param[@name='Data_Signif']").attrib['value'])
                scname = scname + "(Flight)"
                lcurl = root.find("./What/Param[@name='LightCurve_URL']").attrib['value']
            elif type == n.FERMI_GBM_FIN_POS or type == n.FERMI_GBM_GND_POS:
                signif = float(root.find("./What/Param[@name='Burst_Signif']").attrib['value'])
                lcurl = root.find("./What/Param[@name='LightCurve_URL']").attrib['value']
                if type == n.FERMI_GBM_FIN_POS:
                    scname = scname + "(Final)"
                    mapurl = root.find("./What/Param[@name='LocationMap_URL']").attrib['value']
                else:
                    scname = scname + "(Ground)"
            elif type == n.FERMI_GBM_SUBTHRESH:
                signif = 0.
                scname = scname + "(Subthreshold)"
                lcurl = root.find("./What/Param[@name='LC_URL']").attrib['value']
                mapurl = root.find("./What/Param[@name='Map_URL']").attrib['value']
        else:
            if trigID == -1:
                tjd = root.find("./What/Param[@name='Burst_TJD']").attrib['value']
                sod = root.find("./What/Param[@name='Burst_SOD']").attrib['value'].split('.')[0]
                sod = "%05d" % int(float(sod))
                trigID = int(tjd+sod)
            lcurl = "https://gcn.gsfc.nasa.gov/ipn/gcn_ipn_raw.html"
 
        if test and type != n.FERMI_GBM_SUBTHRESH:
            if use_tests:
                msg = scname + " Test Alert"
                print(time.ctime(time.time()), msg+" message")
                print("\t["+description+"]")
                show_alert_warn("Attention! "+msg,600)
        else:    
            msg = scname+" just found a Transient or GRB "
            if trigID > 0:
                msg = msg + str(trigID)
            if err > 0.0001 or signif>0.1:
                msg = msg + " ("
                if err>0.0001:
                    elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Position2D/Value2/C1")
                    ra  = float(elm.text)
                    elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Position2D/Value2/C2")
                    dec = float(elm.text)
                    obj = "RA="+hour2hms(ra/15.)+",Dec="+degr2dms(dec)
                    msg = msg + obj +(",ErrBox=%gdegr" % (err))
                if signif>0.1:
                    if err>0.0001:
                        msg = msg + ","                    
                    msg = msg + ("%gsigma" % (signif))
                msg = msg + ")"
            print(time.ctime(time.time()), msg+" at "+str(dtime))
            if lcurl.startswith('http'):
                print("LightCurveURL:" + lcurl)
            if mapurl.startswith('http'):
                print("MapURL:" + mapurl)
            show_alert_warn("Attention! "+msg,600)

    elif type == n.AGILE_TRANS or type == n.SWIFT_BAT_TRANS or \
         type == n.FERMI_GBM_TRANS or type == n.FERMI_LAT_TRANS:
        trigID = int(root.find("./What/Param[@name='TrigID']").attrib['value'])
        elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Time/TimeInstant/ISOTime")
        isotime = elm.text.strip()
        dtime = datetime.datetime.strptime(isotime, "%Y-%m-%dT%H:%M:%S.%f")
        signif = 0.
        if type == n.AGILE_TRANS: scname = "SuperAGILE"
        elif type == n.SWIFT_BAT_TRANS:
            scname = "Swift-BAT"
            signif = float(root.find("./What/Param[@name='Rate_Signif']").attrib['value'])
            if signif < 0.1:
                signif = float(root.find("./What/Param[@name='Image_Signif']").attrib['value'])
        else: scname = "Fermi"
        msg = scname+" Transient "+str(trigID)
        if signif>0.1:
            msg = msg + (" (%gsigma)" % (signif)) 
        msg = msg + ", but not a GRB"
        print(time.ctime(time.time()), msg+" at "+str(dtime))
        if trigID != grb_ID:
            show_alert_warn(msg,600)
        else: 
            elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Position2D/Value2/C1")
            ra  = float(elm.text)
            elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Position2D/Value2/C2")
            dec = float(elm.text)
            elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Position2D/Error2Radius")
            err = float(elm.text)
            obj = "RA="+hour2hms(ra/15.)+" Dec="+degr2dms(dec)
            print("trigger: ID=%d %s, ErrBox=%g' Signif=%gsigma at %s" % (trigID, obj, err*60., signif, dtime))
            show_alert_gcn(ra/15.,dec,scname+"_Transient_"+str(trigID),msg,3600.,err-0.0001)
            
    elif type == n.SIMBADNED or type == n.SUZAKU_LC:
        if type == n.SIMBADNED: scname = "SIMBAD-NED"
        elif type == n.SUZAKU_LC: scname = "Suzaku WAM"
        print(time.ctime(time.time()), scname+" GRB message")

#---- Test messages for practicing or debugging purpose ------------------------------------------
    elif type == n.TEST_COORDS or type == n.HETE_TEST or type == n.MAXI_TEST or\
         type == n.AGILE_GRB_POS_TEST or type == n.SWIFT_BAT_GRB_POS_TEST or \
         type == n.FERMI_GBM_POS_TEST or type == n.FERMI_LAT_POS_TEST:  
        trigID = int(root.find("./What/Param[@name='TrigID']").attrib['value'])
        elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Time/TimeInstant/ISOTime")
        isotime = elm.text.strip()
        dtime = datetime.datetime.strptime(isotime, "%Y-%m-%dT%H:%M:%S.%f")
        elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Position2D/Value2/C1")
        ra  = float(elm.text)
        elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Position2D/Value2/C2")
        dec = float(elm.text)
        elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Position2D/Error2Radius")
        err = float(elm.text)
        signif = 0.
        if type == n.HETE_TEST: scname = "HETE"
        elif type == n.MAXI_TEST: scname = "MAXI"
        elif type == n.AGILE_GRB_POS_TEST:
            scname = "SuperAGILE"
            signif = float(root.find("./What/Param[@name='SignifX']").attrib['value'])
            signif += float(root.find("./What/Param[@name='SignifY']").attrib['value'])
            signif /= 2.
        elif type == n.SWIFT_BAT_GRB_POS_TEST:
            scname = "Swift"
            signif = float(root.find("./What/Param[@name='Rate_Signif']").attrib['value'])
        elif type == n.TEST_COORDS: scname = "Coordinates"
        else:
            scname = "Fermi"
            if type == n.FERMI_GBM_POS_TEST:
                signif = float(root.find("./What/Param[@name='Data_Signif']").attrib['value'])
        if use_tests:
            msg = scname + " Test Position"
            if signif>0.1:
                msg = msg + (" (%gsigma)" % (signif)) 
            print(time.ctime(time.time()), msg+" message")
            print("\t["+description+"]")
            print("Test trigger: ID=%d RA=%gh Dec=%gdgr, ErrBox=%g' Signif=%gsigma at %s" % (trigID, ra/15., dec, err*60., signif, dtime))
            show_alert_gcn(ra/15.,dec,scname+"_GRB_"+str(trigID),msg,3600.,err)
#----------------------------------------------------------------------------------

    elif type == n.SWIFT_BAT_GRB_POS_NACK or type == n.SWIFT_XRT_CENTROID or \
         type == n.SWIFT_UVOT_POS_NACK: 
        trigID = int(root.find("./What/Param[@name='TrigID']").attrib['value'])
        if type == n.SWIFT_BAT_GRB_POS_NACK: scname = "Swift-BAT"
        elif type == n.SWIFT_XRT_CENTROID: scname = "Swift-XRT"
        elif type == n.SWIFT_UVOT_POS_NACK: scname = "Swift-UVOT"
        msg = scname + " software was not able to find a point source location for GRB "+str(trigID)
        print(time.ctime(time.time()), msg)
        
    elif type == n.COINCIDENCE:
        type1 = int(root.find("./What/Param[@name='Trigger1_Type']").attrib['value'])
        trig1ID = int(root.find("./What/Param[@name='Trigger1_TrigNum']").attrib['value'])
        type2 = int(root.find("./What/Param[@name='Trigger2_Type']").attrib['value'])
        trig2ID = int(root.find("./What/Param[@name='Trigger2_TrigNum']").attrib['value'])
        msg = "Coincidence between Type%d(ID=%d) Type%d(ID=%d) events" % (type1,trig1ID,type2,trig2ID) 
        print(time.ctime(time.time()), msg)
        gcn_save_xml(type,payload)
        
    elif type == n.AMON_ICECUBE_COINC or type == n.AMON_ICECUBE_HESE or \
         type == n.AMON_ICECUBE_TEST or type == n.AMON_ICECUBE_EHE or \
         type == n.AMON_ICECUBE_GOLD or type == n.AMON_ICECUBE_BRONZE or \
         type == n.AMON_HAWC_BURST_MONITOR or type == n.AMON_GAMMA_NU_COINC or \
         type == n.AMON_ICECUBE_CASCADE:
        scname = "AMON IceCube "
        try:
            run_id = root.find("./What/Param[@name='run_id']").attrib['value']
        except AttributeError:
            try:
                run_id = root.find("./What/Param[@name='Run_ID']").attrib['value']
            except AttributeError:
                run_id = "0"
        try:
            event_id = root.find("./What/Param[@name='event_id']").attrib['value']
        except AttributeError:
            try:
                event_id = root.find("./What/Param[@name='Event_ID']").attrib['value']
            except AttributeError:
                event_id = "0"
        trigID = int(run_id+event_id)
        if type == n.AMON_ICECUBE_COINC: scname = scname + "(COINC)"
        elif type == n.AMON_ICECUBE_HESE: scname = scname + "(HESE)"
        elif type == n.AMON_ICECUBE_EHE: scname = scname + "(EHE)"
        elif type == n.AMON_ICECUBE_GOLD: scname = scname + "(Track Gold)"
        elif type == n.AMON_ICECUBE_BRONZE: scname = scname + "(Track Bronze)"
        elif type == n.AMON_HAWC_BURST_MONITOR: scname = "AMON HAWC Burst Monitor"
        elif type == n.AMON_GAMMA_NU_COINC: scname = "NU EM coincidences"
        elif type == n.AMON_ICECUBE_CASCADE: scname = scname + "(Cascade)"
        if type == n.AMON_ICECUBE_TEST:
            test = True
        else:
            test = False
        if test:
            scname = scname + " test"
        try:
            signif = float(root.find("./What/Param[@name='signalness']").attrib['value'])
        except AttributeError:
            signif = None
        try:
            elem = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Position2D/Value2/C1")
            ra  = float(elem.text)
            elem = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Position2D/Value2/C2")
            dec = float(elem.text)
            elem = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Position2D/Error2Radius")
            errad = float(elem.text)
            if errad<0.0001: errad = None
        except AttributeError:
            ra  = None
            dec  = None
            errad  = None
        if not test or use_tests:
            msg = scname + " event "+str(trigID)
            if signif!=None: msg += "(signalness="+str(signif)+")"
            print(time.ctime(time.time()), msg)
            if ra!=None and dec!=None and errad!=None and signif!=None:
                show_alert_gcn(ra/15.,dec,scname+" "+str(trigID),msg,3600.,errad)
            else:    
                show_alert_warn(msg,600)
        if not test:
            gcn_save_xml(type,payload)
            
    elif type == n.LVC_PRELIMINARY or type == n.LVC_INITIAL or \
         type == n.LVC_UPDATE or type == n.LVC_COUNTERPART or \
         type == n.LVC_RETRACTION or type == n.LVC_TEST:
        scname = "LIGO-Virgo Collaboration "
        graceID = root.find("./What/Param[@name='GraceID']").attrib['value']
        if graceID.upper().startswith('MS') or graceID.upper().startswith('TS'):
            test = True
        else:
            test = False
        if  type != n.LVC_RETRACTION and not test:
            try:
                instrume = root.find("./What/Param[@name='Instruments']").attrib['value']
                scname = scname + instrume
            except AttributeError:
                pass
        if type == n.LVC_PRELIMINARY: scname = scname + "(PRELIMINARY)"
        elif type == n.LVC_INITIAL: scname = scname + "(INITIAL)"
        elif type == n.LVC_UPDATE: scname = scname + "(UPDATE)"
        elif type == n.LVC_COUNTERPART: scname = scname + "(COUNTERPART)"
        elif type == n.LVC_RETRACTION: scname = scname + "(RETRACTION)"
        if test:
            scname = scname + " test"
        if not test or use_tests:
            msg = scname + " event"
            print(time.ctime(time.time()), msg)
            show_alert_warn(msg,600)
            if not graceID.upper().startswith('TS'):
                url = root.find("./What/Param[@name='EventPage']").attrib['value']
                print("URL: " + url)
        if not test:
            gcn_save_xml(type,payload)

    elif type == n.KONUS_LC or type == n.SWIFT_XRT_LC or \
         type == n.SWIFT_BAT_GRB_LC or type == n.AGILE_MCAL_ALERT or \
         type == n.CALET_GBM_FLT_LC or type == n.CALET_GBM_GND_LC:
        if type == n.KONUS_LC:
            scname = "KONUS Wind"
        elif type == n.SWIFT_BAT_GRB_LC:
            scname = "Swift-BAT Lightcurve"
        elif type == n.SWIFT_XRT_LC:
            scname = "Swift-XRT Lightcurve"
        elif type == n.AGILE_MCAL_ALERT:
            scname = "SuperAGILE MCAL"
        else:
            scname = "CALET/ISS Lightcurve "
            if type == n.CALET_GBM_FLT_LC:
                scname = scname + "(Flight)"
            elif type == n.CALET_GBM_GND_LC:
                scname = scname + "(Ground)"
        trigID = int(root.find("./What/Param[@name='TrigID']").attrib['value'])
        elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Time/TimeInstant/ISOTime")
        isotime = elm.text.strip()
        dtime = datetime.datetime.strptime(isotime, "%Y-%m-%dT%H:%M:%S.%f")
        if type == n.SWIFT_XRT_LC:
            url = root.find("./What/Param[@name='Light_Curve_Jpeg']").attrib['value']
        elif type == n.AGILE_MCAL_ALERT:
            url = root.find("./What/Param[@name='LightCurve_URL']").attrib['value']
        else:
            url = root.find("./What/Param[@name='Light_Curve']").attrib['value']
        msg = scname + " event"
        print(time.ctime(time.time()), msg+" ID="+str(trigID)+" at "+str(dtime))
        print("LightCurveURL: " + url)
        if type == n.CALET_GBM_GND_LC:
            show_alert_warn(msg,600)
            gcn_save_xml(type,payload)

    elif type == n.SNEWS:
        scname = "SNEWS"
        trigID = int(root.find("./What/Param[@name='TrigID']").attrib['value'])
        elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Time/TimeInstant/ISOTime")
        isotime = elm.text.strip()
        dtime = datetime.datetime.strptime(isotime, "%Y-%m-%dT%H:%M:%S.%f")
        ptest = root.find("./What/Group[@name='Trigger_ID']/Param[@name='Test']").attrib['value'].lower().startswith('true')
        rtest = role.startswith('test')
        test = ptest and rtest
        if test:
            scname = scname + " test"
        if not test or use_tests:
            msg = scname + " event"
            print(time.ctime(time.time()), msg+" ID="+str(trigID)+" at "+str(dtime))
            print("\t["+description+"]")
            show_alert_warn(msg,600)
        if not test:
            gcn_save_xml(type,payload)
        
    elif type == n.IPN_POS or \
         type == n.INTEGRAL_WAKEUP or type == n.INTEGRAL_REFINED or \
         type == n.INTEGRAL_OFFLINE or type == n.INTEGRAL_WEAK or \
         type == n.AGILE_GRB_WAKEUP or type == n.AGILE_GRB_GROUND or \
         type == n.AGILE_GRB_REFINED or \
         type == n.FERMI_LAT_GND or type == n.FERMI_LAT_OFFLINE or \
         type == n.SWIFT_BAT_GRB_POS_ACK or type == n.SWIFT_XRT_POSITION or \
         type == n.SWIFT_BAT_QL_POS or type == n.SWIFT_BAT_SLEW_POS or \
         type == n.SWIFT_BAT_SUB_THRESHOLD or type == n.SWIFT_UVOT_POS or \
         type == n.GRB_CNTRPART:
        trigID = int(root.find("./What/Param[@name='TrigID']").attrib['value'])
        elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Time/TimeInstant/ISOTime")
        isotime = elm.text.strip()
        dtime = datetime.datetime.strptime(isotime, "%Y-%m-%dT%H:%M:%S.%f")
        elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Position2D/Value2/C1")
        ra  = float(elm.text)
        elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Position2D/Value2/C2")
        dec = float(elm.text)
        elm = xml_find_path(root,"./WhereWhen/ObsDataLocation/ObservationLocation/AstroCoords/Position2D/Error2Radius")
        err = float(elm.text)
        obj = "RA="+hour2hms(ra/15.)+" Dec="+degr2dms(dec)

        is_GRB = 'true'
        is_not_GRB = 'false'
        signif = 0.
        if type == n.INTEGRAL_WAKEUP or type == n.INTEGRAL_REFINED or \
           type == n.INTEGRAL_OFFLINE or type == n.INTEGRAL_WEAK:
            test = root.find("./What/Group[@name='Test_mpos']/Param[@name='Test_Notice']").attrib['value'].lower().startswith('true')
            is_not_GRB = root.find("./What/Group[@name='Test_mpos']/Param[@name='Def_NOT_a_GRB']").attrib['value']
            signif = float(root.find("./What/Param[@name='Inten_Sigma']").attrib['value'])
        elif type == n.AGILE_GRB_WAKEUP or type == n.AGILE_GRB_GROUND or \
             type == n.AGILE_GRB_REFINED:               
            is_GRB = root.find("./What/Group[@name='Trigger_ID']/Param[@name='GRB_Identified']").attrib['value']
            is_not_GRB = root.find("./What/Group[@name='Trigger_ID']/Param[@name='Def_NOT_a_GRB']").attrib['value']
        elif type == n.FERMI_LAT_POS_UPD or type == n.FERMI_LAT_GND or \
             type == n.FERMI_LAT_OFFLINE:
            is_not_GRB = root.find("./What/Group[@name='Trigger_ID']/Param[@name='Def_NOT_a_GRB']").attrib['value']
        elif type == n.SWIFT_BAT_GRB_POS_ACK or type == n.SWIFT_BAT_SUB_THRESHOLD or \
             type == n.SWIFT_BAT_SLEW_POS:
            is_GRB = root.find("./What/Group[@name='Solution_Status']/Param[@name='GRB_Identified']").attrib['value']
            is_not_GRB = root.find("./What/Group[@name='Solution_Status']/Param[@name='Def_NOT_a_GRB']").attrib['value']
            signif = float(root.find("./What/Param[@name='Rate_Signif']").attrib['value'])
            if signif < 0.1:
                signif = float(root.find("./What/Param[@name='Image_Signif']").attrib['value'])
        elif type == n.SWIFT_BAT_QL_POS:
            signif = float(root.find("./What/Param[@name='Rate_Signif']").attrib['value'])
        elif type == n.SWIFT_UVOT_POS or type == n.SWIFT_XRT_POSITION:
            is_not_GRB = root.find("./What/Group[@name='Solution_Status']/Param[@name='Def_NOT_a_GRB']").attrib['value']
            if type == n.SWIFT_XRT_POSITION:
                signif = float(root.find("./What/Param[@name='Det_Signif']").attrib['value'])
            else:
                is_GRB = root.find("./What/Group[@name='Solution_Status']/Param[@name='GRB_Identified']").attrib['value']
        elif type == n.IPN_POS:
            if root.find("./What/Group[@name='Trigger_ID']/Param[@name='Definite_NOT_GRB']").attrib['value'].startswith('true'):
                is_not_GRB = 'true'
        if type == n.INTEGRAL_WAKEUP: scname = "INTEGRAL(Wakeup)"
        elif type == n.INTEGRAL_REFINED: scname = "INTEGRAL(Refined)"
        elif type == n.INTEGRAL_OFFLINE: scname = "INTEGRAL(Offline)"
        elif type == n.INTEGRAL_WEAK: scname = "INTEGRAL(Weak)"
        elif type == n.AGILE_GRB_WAKEUP: scname = "SuperAGILE(Wakeup)"
        elif type == n.AGILE_GRB_GROUND: scname = "SuperAGILE(Ground)"
        elif type == n.AGILE_GRB_REFINED: scname = "SuperAGILE(Refined)"
        elif type == n.FERMI_LAT_GND: scname = "Fermi-LAT(Ground)"
        elif type == n.FERMI_LAT_OFFLINE: scname = "Fermi-LAT(Offline)"
        elif type == n.SWIFT_BAT_QL_POS or \
             type == n.SWIFT_BAT_GRB_POS_ACK: scname = "Swift-BAT"
        elif type == n.SWIFT_BAT_SUB_THRESHOLD: scname = "Swift-BAT(SubThr)"
        elif type == n.SWIFT_BAT_SLEW_POS: scname = "Swift-BAT(Slew)"
        elif type == n.SWIFT_XRT_POSITION: scname = "Swift-XRT"
        elif type == n.SWIFT_UVOT_POS: scname = "Swift-UVOT"
        elif type == n.GRB_CNTRPART: scname = "Counterpart"
        if test:
            if use_tests:
                msg = scname + " Test Position"
                if signif>0.1:
                    msg = msg + (" (%gsigma)" % (signif)) 
                print(time.ctime(time.time()), msg+" message")
                print("\t["+description+"]")
                print("Test trigger: ID=%d RA=%gh Dec=%gdgr, ErrBox=%g' Signif=%gsigma at %s" % (trigID, ra/15., dec, err*60., signif, dtime))
                show_alert_gcn(ra/15.,dec,scname+"_GRB_"+str(trigID),msg,3600.,err)
        elif is_GRB == 'true' and is_not_GRB == 'false':
            if err > 0.0001:
                msg = scname + " GRB Position"
                if type == n.GRB_CNTRPART:
                    msg = scname + " to GRB-"+str(trigID)+"(from "+isotime+")"
                if signif>0.1:
                    msg = msg + (" (%gsigma)" % (signif)) 
                print(time.ctime(time.time()), msg+" message")
#                print "\t["+description+"]"
                print("trigger: ID=%d %s, ErrBox=%g' Signif=%gsigma at %s" % (trigID, obj, err*60., signif, dtime))
                show_alert_gcn(ra/15.,dec,scname+"_GRB_"+str(trigID),msg,36000.,err)
                grb_ID = trigID
            else:
                msg = scname + " GRB message (but no ErrBox?)"
                print(time.ctime(time.time()), msg)
                print("\t["+description+"]")
                print("trigger: ID=%d %s, ErrBox=%g' Signif=%gsigma at %s" % (trigID, obj, err*60., signif, dtime))
                show_alert_warn(msg,600)
        else:
            msg = scname + " Position trigger("+obj+") Transient "+str(trigID)+", but not a GRB"
            print(time.ctime(time.time()), msg + " at "+str(dtime))
            show_alert_warn(msg,1800)
    else:
        if type != n.INTEGRAL_POINTDIR and type != n.AGILE_POINTDIR and \
           type != n.SWIFT_POINTDIR and type != n.SWIFT_CURR_POINTDIR and \
           type != n.FERMI_POINTDIR and \
           type != n.SWIFT_BAT_GRB_LC and type != n.SWIFT_BAT_SCALEDMAP and \
           type != n.SWIFT_FOM_OBS and type != n.SWIFT_SC_SLEW and \
           type != n.SWIFT_XRT_SPECTRUM and type != n.SWIFT_XRT_IMAGE and \
           type != n.SWIFT_UVOT_DBURST and type != n.SWIFT_UVOT_FCHART and \
           type != n.SWIFT_XRT_SPECTRUM_PROC and type != n.SWIFT_XRT_IMAGE_PROC and \
           type != n.SWIFT_UVOT_DBURST_PROC and type != n.SWIFT_UVOT_FCHART_PROC and \
           type != n.SWIFT_TOO_FOM and type != n.SWIFT_TOO_SC_SLEW and \
           type != n.SWIFT_BAT_SUBSUB and type != n.SWIFT_BAT_KNOWN_SRC and \
           type != n.SWIFT_BAT_MONITOR and type != n.FERMI_LAT_MONITOR and \
           type != n.FERMI_SC_SLEW and type != n.MOA:
            print(time.ctime(time.time()),"Unhandled message: Type=%d" % type)
            print("\t["+description+"]")
            gcn_save_xml(type,payload)

    sys.stdout.flush()

try:
  from lxml import etree
except ImportError:
    import xml.etree.cElementTree as etree

def xml_find_tag(root,tagname):
    ret=None
    for elm in root.iter():
        if elm.tag.endswith(tagname):
            ret = elm
    return ret

def xml_find_path(root,path):
    tags = path.split('/')
    elm = root
    for i in range(len(tags)):
        tag = tags[i]
        if tag == '.':
            if i == 0: elm = root
        else:
            nelm = None
            for child in elm:
                if child.tag.endswith(tag):
                    nelm = child
                    break
            elm = nelm
            if elm == None:
                print("Can't find tag '"+tag+"' in '"+path+"'")
                break
    return elm


if __name__ == '__main__':
    from optparse import OptionParser

    parser = OptionParser(usage="usage: %prog [options] arg")
    parser.add_option('-H', '--host', help='Host to connect', action='store', dest='host', default=gcn_host3)
    parser.add_option('-p', '--port', help='Port to connect', action='store', dest='port', type='int', default=gcn_port)

    (options,args) = parser.parse_args()

    gcn.listen(host=options.host, port=options.port, handler=gcn_handler)
