#!/usr/bin/python

"""
Show GCN object and telescope current positions at sky
"""
__author__ = "Vladimir Shergin <http://www.sao.ru/hq/vsher/>"

import os, sys, re, socket
import http.client, urllib.request, urllib.parse, urllib.error
from xmlrpc.client import ServerProxy, Boolean, ProtocolError, Fault, Error
from threading import Timer
import math
import tkinter
from gcn_ztcs import hour2hms, degr2dms

skyview=False
hdgrid=False
xy_size=500
saolong = 41.4414375
saolat = 43.65352769
cos_fi=0.7235272793
sin_fi=0.6902957888
tname="Zeiss"           # Name of SAO's telescope: 'BTA' or 'Zeiss'
thost="ann.sao.ru"      # Network name of TCS computer 
pswd = "ruser"          # Password (for level>=1)
is_bta = False          # default - Zeiss-1000
bhost="tb.sao.ru"       # BTA TCS default host
#bhost="acs7.sao.ru"    # BTA TCS default host
zhost="ztcs.sao.ru"     # Zeiss-1000 TCS default host
zport=8088
bpsw="selsin"
zuser="user"
zpsw="ztcs"
zserver = None
tmode = ""
tra = 0.0
tdec = 0.0
taz = 0.0
tzd = 0.0
oname="Object"
ora = 0.0
odec = 0.0
oaz = 0.0
ozd = 0.0

def bta_tcs_state():
    global thost,pswd,level
    global stime, mjd, tmode, tra, tdec, taz, tzd
    server = http.client.HTTPConnection(thost, timeout=3)
    headers = {"Content-type":"application/x-www-form-urlencoded","Accept":"text/plain"}
    params = urllib.parse.urlencode({"Pass": pswd, "Cmd": "Get"})
    level=0
    try:
        server.request("POST", "/tcs/ctrl/bta_send.cgi", params, headers)
        resp = server.getresponse()
        if resp.status != 200:
            print(resp.status, resp.reason)
            return False
        state = "Err"
        ll = resp.read().decode().split('\n')
        for i in range(len(ll)):
            l = str(ll[i])
            if l.count('=')==1:
                nv = l.split('=')
                nam = str(nv[0])
                val = str(nv[1])
                if   nam == "Level":   level = int(val)
                elif nam == "Stime":   stime = float(val)%24
                elif nam == "MJD":     mjd = float(val)
                elif nam == "TelMode": tmode = val
                elif nam == "RA":      tra = float(val)
                elif nam == "Dec":     tdec = float(val)
                elif nam == "A":       taz = float(val)
                elif nam == "Z":       tzd = float(val)
                elif nam == "State":   state = val
#            else:
#                print(l)
        server.close()
        del server
        server = None
        if state == "Ok":
            return True
        else:
            return False
    except socket.error as serr:
        print("Network Error: %s" % serr.strerror)
    except http.client.HTTPException as err:
        print("HTTP error %s" % str(err))
    try:
        server.close()
    finally:
        del server
        server = None
        return False

def zeiss_tcs_ok():
    "Internal function"
    global zserver, level
    return isinstance(zserver,ServerProxy) and level>0

def zeiss_tcs_state():
    global zserver,thost,zport,zuser,pswd,level
    global stime, tmode, tra, tdec, taz, tzd
    if not zeiss_tcs_ok():
        if not isinstance(zserver,ServerProxy):
            server_url = "http://"+zuser+":"+pswd+"@"+thost+":"+str(zport)
            zserver = ServerProxy(server_url)
        level=0
        try:
            info = zserver.Server.info()
            n = info.rfind("Level=")
            if n>0:
               level = int(info[n+6])
        except socket.error as serr:
            print("Zeiss TCS Network Error: %s" % serr[1])
            try:
                zserver.__close()
            finally:
                del zserver
                zserver = None
                return False
    if level<1: return False
    try:
        reqst = "TelMeanST Status TelAppRA TelAppDecl TelObsAzim TelObsZD"
        repl = re.split("\s+",zserver.Telescope.getStatusString(reqst))
        ret=True
        for i in range(0,len(repl)):
            if repl[i].count('=')==1:
                l=repl[i].split('=')
                nam = l[0]
                val = l[1]
                if nam == "TelMeanST":    stime = float(val)/3600
                elif nam == "Status":     tmode = val
                elif nam == "TelAppRA":   tra = float(val)/3600*15
                elif nam == "TelAppDecl":  tdec = float(val)/3600
                elif nam == "TelObsAzim": taz = float(val)/3600
                elif nam == "TelObsZD":   tzd = float(val)/3600
                else:
                    print(repl[i]+"???")
                    ret=False
            else:
                print(repl[i]+"???")
                ret=False
        return ret
    except socket.error as serr:
        print("Network Error: %s" % serr.strerror)
    except ProtocolError as err:
        print("A protocol error %d: %s" % err.errcode, err.errmsg)
    except Fault as err:
        print("Fault %d: %s" % err.faultCode, err.faultString)
    except Error as v:
        print("ERROR", v)
    try:
        zserver.__close()
    finally:
        del zserver
        zserver = None
        return False


def calc_AZ(RA, Dec, stime):
    global oha
    st = stime*15.
    ha = st - RA
    if(ha < 0.): ha += 360.
    if(ha > 360.): ha -= 360.
    oha = ha/15.
    azd=hd2az(ha, Dec)
    return azd

def hd2az(HA, Dec):
    rha = HA/180.*math.pi
    rdec = Dec/180.*math.pi
    sin_h = math.sin(rha)
    cos_h = math.cos(rha)
    sin_d = math.sin(rdec)
    cos_d = math.cos(rdec)
    cos_z = cos_fi * cos_d * cos_h + sin_fi * sin_d
    rz = math.acos(cos_z)
    y = cos_d * sin_h;
    x = cos_d * sin_fi * cos_h - cos_fi * sin_d
    ra = math.atan2(y, x)
    z = rz/math.pi*180.
    a = ra/math.pi*180.
    return [a,z]

def az2xy(a, z):
    global xy_size,skyview
    R=x0=y0=xy_size/2
    ar = a/180.*math.pi
    if(skyview):
        if z < 90:
            r=R*z/90
        else:
            r=R*(180-z)/90
        x = int(r*math.sin(ar)+x0+0.5)
        y = int(r*math.cos(ar)+y0+0.5)
    else:
        r=R*math.sin(z/180.*math.pi);
        x = int( r*math.sin(ar)+x0+0.5)
        y = int(-r*math.cos(ar)+y0+0.5)
    return [x,y]

top = None
tmr = None
new_top=False

def tel_obj_show():
    global is_bta,top,frm,btns,cvs,rbt1,rbt2,new_top,hdgrid,skyview
    if top==None:
        top = tkinter.Toplevel()
        new_top=True
    frm = tkinter.LabelFrame(top, height=xy_size+10, width=xy_size+10,labelanchor=tkinter.NW, text="Frame1", font=("Nimbus Roman",12,"italic"))
    btns = tkinter.Frame(top, height=30, width=xy_size+10)
    bt = tkinter.Button(btns, text ="Close", command=tel_obj_quit)
    bt.pack(side=tkinter.RIGHT)
    rbt1 = tkinter.IntVar()
    rbt2 = tkinter.IntVar()
    rbt1.set(2)
    rbt2.set(1)
    btns1 = tkinter.Frame(btns, height=30, bd=2, relief=tkinter.RAISED)
    btns2 = tkinter.Frame(btns, height=30, bd=2, relief=tkinter.RAISED)
    lab1 = tkinter.Label(btns1, text="Grid mode:")
    rbthd = tkinter.Radiobutton(btns1, text="HA/Dec", variable=rbt1, value=1, command=az_or_hd)
    rbtaz = tkinter.Radiobutton(btns1, text="Az/ZD", variable=rbt1, value=2, command=az_or_hd)
    if hdgrid: rbthd.select()
    else: rbtaz.select()
    lab1.pack(side=tkinter.LEFT)
    rbthd.pack(side=tkinter.LEFT, anchor=tkinter.E)
    rbtaz.pack(side=tkinter.RIGHT, anchor=tkinter.W)
    lab2 = tkinter.Label(btns2, text="View like:")
    rbtbta = tkinter.Radiobutton(btns2, text="BTA", variable=rbt2, value=1, command=bta_or_sky)
    rbtsky = tkinter.Radiobutton(btns2, text="AllSky", variable=rbt2, value=2, command=bta_or_sky)
    if skyview: rbtsky.select()
    else: rbtbta.select()
    lab2.pack(side=tkinter.LEFT)
    rbtbta.pack(side=tkinter.LEFT, anchor=tkinter.E)
    rbtsky.pack(side=tkinter.RIGHT, anchor=tkinter.W)
    btns1.pack(side=tkinter.LEFT)
    btns2.pack(side=tkinter.LEFT)
    cvs = None
    sky_or_bta_draw()
    cvs.pack()
    frm.pack()
    if is_bta: tnam="BTA"
    else: tnam="Zeiss1000"
    frm.configure(text=tnam+" and "+oname)
    btns.pack(side=tkinter.BOTTOM,expand=True,fill=tkinter.X)
    if new_top:
        top.title("Showing "+tnam+" and "+oname)
        top.mainloop()

def bta_or_sky():
    global frm,cvs,rbt2,skyview
    if rbt2.get()==2: skyview=True
    else: skyview=False
    cvs.destroy()
    cvs = None
    sky_or_bta_draw()
    cvs.pack()

def az_or_hd():
    global frm,cvs,rbt1,hdgrid
    if rbt1.get()==1: hdgrid=True
    else: hdgrid=False
    cvs.destroy()
    cvs = None
    sky_or_bta_draw()
    cvs.pack()

def sky_or_bta_draw():
    global is_bta, frm,cvs, xy_size, hdgrid, ora,odec,oha,stime, taz,tzd, oaz,ozd,oname
    if not isinstance(cvs,tkinter.Canvas):
        cvs = tkinter.Canvas(frm, bg="#cceeff", height=xy_size, width=xy_size)
    MyFont1 = ("Times New Roman",12,"italic")
    MyFont2 = ("Times New Roman",8,"italic")
    MyFont3 = ("Times New Roman",10,"bold")

    x0=y0=xy_size/2
    cvs.create_oval( 0, 0, xy_size, xy_size,  width=3, outline="Blue")
    z=5.
    xy=az2xy(90,z)
    x=y=xy[0]
    ir=x-x0
    cvs.create_oval(x0-ir, y0-ir, x0+ir, y0+ir,  width=2, outline="Blue")
    if hdgrid:
        for h in range(0,361,15):
            hh = int(h/15)
            azd=hd2az(h, 90)
            xy=az2xy(azd[0], azd[1])
            for d in range(89,-45,-1):
                azd=hd2az(h, d)
                if azd[1] > 90: break
                xy1=az2xy(azd[0], azd[1])
                if d<80 or (d<85 and hh%2==0) or (d<=89 and hh%6==0):
                    cvs.create_line(xy[0], xy[1], xy1[0], xy1[1], fill="DarkSlateBlue")
                if d==0 and hh<24:
                    cvs.create_text(xy1[0]+5, xy1[1]+5, text=str(hh), font=MyFont2, fill="DarkGreen")
                if hh==0 and d%10==0:
                    cvs.create_text(xy1[0]-5, xy1[1]-5, text=str(d), font=MyFont2, fill="DarkGreen")
                xy=xy1
        for d in [85]+list(range(80,-50,-10)):
            azd=hd2az(0, d)
            xy=az2xy(azd[0], azd[1])
            z=azd[1]
            for h in range(1,180,1):
                azd=hd2az(h, d)
                if z>89 and azd[1]>90: break
                xy1=az2xy(azd[0], azd[1])
                cvs.create_line(xy[0], xy[1], xy1[0], xy1[1], fill="DarkSlateBlue")
                z=azd[1]
                xy=xy1
            azd=hd2az(360, d)
            xy=az2xy(azd[0], azd[1])
            z=azd[1]
            for h in range(359,180,-1):
                azd=hd2az(h, d)
                if z>89 and azd[1]>90: break
                xy1=az2xy(azd[0], azd[1])
                cvs.create_line(xy[0], xy[1], xy1[0], xy1[1], fill="DarkSlateBlue")
                z=azd[1]
                xy=xy1
    else:
        for z in range(10,90,10):
            xy=az2xy(90,z)
            x=y=xy[0]
            ir=x-x0
            cvs.create_oval(x0-ir, y0-ir, x0+ir, y0+ir,  width=1, outline="DarkSlateBlue")
            xy=az2xy(0,z)
            y=xy[1]
            cvs.create_text(x0+5, y+5, text=str(z), font=MyFont2, fill="DarkGreen")
        for a in range(0,360,10):
            if a%90==0: z=0
            elif a%30==0: z=5
            else: z=10
            xy=az2xy(a,z)
            z=90
            xy1=az2xy(a,z)
            cvs.create_line(xy[0], xy[1], xy1[0], xy1[1], fill="DarkSlateBlue")
            if a%20 == 0:
                y = xy1[1]
#                if y < 5: y += 10;
                if y < y0: y += 5;
                else: y -= 5;
                if a<181:
                    a1 = a
                    x = xy1[0]-10
                else:
                    a1 = a - 360
                    x = xy1[0]+13
                cvs.create_text(x, y, text=str(a1), font=MyFont2, fill="DarkGreen")
    azd=hd2az(0, odec)
    a=azd[0]
    z=azd[1]
    xy=az2xy(a, z)
    for h in range(2,361,2):
        azd=hd2az(h, odec)
        xy1=az2xy(azd[0], azd[1])
        if(z>90 or azd[1]>90): col="Black"
        else: col="Orange"
        cvs.create_line(xy[0], xy[1], xy1[0], xy1[1], fill=col)
        xy=xy1
        z=azd[1]

    xy=az2xy(taz,tzd)
    x=xy[0]
    y=xy[1]
    col="Red"
    cvs.create_line(x-1, y, x-10, y, width=2, fill=col)
    cvs.create_line(x+1, y, x+10, y, width=2, fill=col)
    cvs.create_line(x, y-1, x, y-10, width=2, fill=col)
    cvs.create_line(x, y+1, x, y+10, width=2, fill=col)
    cvs.create_oval(x-11, y-11, x+10, y+10,  width=1, outline=col)
    if is_bta: tnam="BTA"
    else: tnam="Zeiss"
    if x < 40:
        cvs.create_text(x+26, y, text=tnam, font=MyFont1, fill=col)
    else:
        cvs.create_text(x-26, y, text=tnam, font=MyFont1, fill=col)
    xy = az2xy(oaz, ozd)
    x=xy[0]
    y=xy[1]
    col1="Black"
    col2="DarkGray"
    if ozd < 89 and ozd > 5:
        col1="Red"
        col2="Yellow"
    cvs.create_line(x-3, y-3, x-10, y-10, width=2, fill=col1)
    cvs.create_line(x+3, y+3, x+10, y+10, width=2, fill=col1)
    cvs.create_line(x-3, y+3, x-10, y+10, width=2, fill=col1)
    cvs.create_line(x+3, y-3, x+10, y-10, width=2, fill=col1)
    cvs.create_line(x-3, y, x-13, y, width=2, fill=col1)
    cvs.create_line(x+3, y, x+13, y, width=2, fill=col1)
    cvs.create_line(x, y-3, x, y-13, width=2, fill=col1)
    cvs.create_line(x, y+3, x, y+13, width=2, fill=col1)
    cvs.create_oval(x-4, y-4, x+4, y+4,  width=2, fill=col2, outline=col1)
    if x < 40: x += 30
    elif x > xy_size-40: x -= 30
    if y < 40: y += 20
    else: y -= -20
    cvs.create_text(x, y, text=oname, font=MyFont2, fill=col)
    cvs.create_text(48, 10, text="A="+degr2dms(oaz), font=MyFont3, fill=col1)
    cvs.create_text(48, 25, text="Z="+degr2dms(ozd), font=MyFont3, fill=col1)
    cvs.create_text(xy_size-50, 10, text="RA="+hour2hms(ora/15.), font=MyFont3, fill=col1)
    cvs.create_text(xy_size-50, 25, text="Dec="+degr2dms(odec), font=MyFont3, fill=col1)
    cvs.create_text(xy_size-40, 37, text="ST="+hour2hms(stime), font=MyFont2, fill=col1)
    cvs.create_text(xy_size-40, 47, text="HA="+hour2hms(oha), font=MyFont2, fill=col1)

def tel_obj_quit():
    global top, frm, btns, tmr, new_top
    if tmr != None:
        tmr.cancel()
        del tmr
        tmr=None
    if new_top:
        top.destroy()
        del top
    elif top!=None:
        if isinstance(btns,tkinter.Frame): btns.destroy()
        if isinstance(frm,tkinter.LabelFrame): frm.destroy()
    top=None

def show_sky_refresh():
    global is_bta, cvs, oaz, ozd, ora, odec, stime, tmr
    if is_bta:
        if not bta_tcs_state():
            print("Can't connect to BTA TCS!")
            sys.stdout.flush()
            tel_obj_quit()
            return
#            sys.exit(0)
    else:
        if not zeiss_tcs_state():
            print("Can't connect to Zeiss1000 TCS!")
            sys.stdout.flush()
            tel_obj_quit()
            return
#            sys.exit(0)
    azd=calc_AZ(ora, odec, stime)
    oaz=azd[0]
    ozd=azd[1]
    cvs.destroy()
    cvs = None
    sky_or_bta_draw()
    cvs.pack()
    del tmr
    tmr = Timer(300.0, show_sky_refresh)
    tmr.start()

def show_sky_start(ra=0.,dec=0.,name="Object",master=None):
    global top,new_top, is_bta, oaz,ozd,ora,odec,oname, stime, tmr
    if top==None:
        if is_bta:
            if not bta_tcs_state():
                print("Can't connect to BTA TCS!")
                sys.stdout.flush()
                return
#                sys.exit(0)
        else:
            if not zeiss_tcs_state():
                print("Can't connect to Zeiss1000 TCS!")
                sys.stdout.flush()
                return
#                sys.exit(0)
        if master!=None:
            new_top=False
        top=master
        oname=name
        ora=ra
        odec=dec
        azd=calc_AZ(ra, dec, stime)
        oaz=azd[0]
        ozd=azd[1]
        tmr = Timer(300.0, show_sky_refresh)
        tmr.start()
        tel_obj_show()

def show_sky_init(telname=None,telhost=None,uname=None,passwd=None):
    global tname,thost,zuser,bpsw,zpsw,pswd, is_bta,skyview,hdgrid
    if telname!=None: tname=telname
    if tname.upper()=="BTA":
        is_bta=True
        skyview=False
        hdgrid=False
    else:
        is_bta=False
        skyview=True
        hdgrid=True
    if telhost!=None:
        if telhost.find('.')>=0:
            thost=telhost
        else:
            thost=telhost+".sao.ru"
    else:
        if is_bta: thost=bhost
        else: thost=zhost
    if uname!=None and not is_bta: zuser=uname
    if passwd!=None: pswd=passwd
    else:
        if is_bta: pswd=bpsw
        else: pswd=zpsw
     

#-------------------------------------------
# Test code
import time
if __name__ == "__main__":
    show_sky_init("BTA","ann",passwd="suser")
    show_sky_start(ra=150.,dec=10.,name="Test")
